/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.speech;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.Destination;
import javax.jms.Session;
import org.robokind.api.common.utils.Listener;
import org.robokind.api.common.utils.TimeUtils;
import org.robokind.api.speech.SpeechEvent;
import org.robokind.api.speech.SpeechRequest;
import org.robokind.api.speech.SpeechService;
import org.robokind.api.speech.utils.SpeechEventNotifier;
import org.robokind.avrogen.speech.SpeechConfigRecord;
import org.robokind.impl.messaging.JMSAvroServiceFacade;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JMSSpeechService extends 
        JMSAvroServiceFacade<SpeechConfigRecord> implements SpeechService{
    private final static Logger theLogger = 
            Logger.getLogger(JMSSpeechService.class.getName());
    private Session mySession;
    private JMSSpeechRequestSender myRequestSender;
    private JMSSpeechEventReceiver myEventReceiver;
    private SpeechEventNotifier mySpeechEventNotifier;
    private String mySpeechServiceId;
    /**
     * Connects to a remote SpeechService through JMS and Avro
     * @param session JMS Session to use
     * @param cmdDest JMS Destination for sending service commands
     * @param errDest JMS Destination for receiving service errors
     * @param requestDest JMS Destination for sending SpeechRequests
     * @param eventDest JMS Destination for receiving SpeechEvents
     * @throws NullPointerException if an argument is null
     * @throws Exception if there is an error connecting
     */
    public JMSSpeechService(String speechServiceId,
            Session session, Destination cmdDest, Destination errDest,
            Destination requestDest, Destination eventDest) throws Exception{
        super(SpeechConfigRecord.class, new SpeechConfigRecord(), 
                session, cmdDest, errDest);
        if(speechServiceId == null ||session == null || 
                requestDest == null || eventDest == null){
            throw new NullPointerException();
        }
        mySession = session;
        myRequestSender = new JMSSpeechRequestSender(mySession, requestDest);
        myRequestSender.setSourceId("MSSAPI");
        myRequestSender.setDestinationId("RobokindSpeech");
        myRequestSender.start();
        mySpeechEventNotifier = new SpeechEventNotifier();
        myEventReceiver = new JMSSpeechEventReceiver(mySession, eventDest);
        myEventReceiver.addMessageListener(mySpeechEventNotifier);
        myEventReceiver.start();
    }
    
    @Override
    public String getSpeechServiceId(){
        return mySpeechServiceId;
    }
    
    @Override
    public void start() throws Exception{
        start(TimeUtils.now());
    }

    @Override
    public void speak(String text) {
        if(myRequestSender == null || text == null){
            throw new NullPointerException();
        }
        theLogger.log(Level.INFO, "Speaking: {0}", text);
        myRequestSender.sendSpeechRequest(text);
    }

    @Override
    public void close() {
        stop(TimeUtils.now());
        myEventReceiver.stop();
        myRequestSender.stop();
    }

    @Override
    public void addRequestListener(Listener<SpeechRequest> listener) {
        if(myRequestSender != null){
            myRequestSender.addMessageListener(listener);
        }
    }

    @Override
    public void removeRequestListener(Listener<SpeechRequest> listener) {
        if(myRequestSender != null){
            myRequestSender.removeMessageListener(listener);
        }
    }

    @Override
    public void addSpeechEventListener(Listener<SpeechEvent> listener) {
        mySpeechEventNotifier.addSpeechEventListener(listener);
    }

    @Override
    public void removeSpeechEventListener(Listener<SpeechEvent> listener) {
        mySpeechEventNotifier.removeSpeechEventListener(listener);
    }
    
}
